from django.test import TestCase
from django.test.utils import override_settings

from rest_framework.reverse import reverse
from rest_framework.test import APIClient

from ovp_projects.models import Project, Apply
from ovp_users.models import User
from ovp_organizations.models import Organization

from collections import OrderedDict

import json
import copy


base_project = {"name": "test project", "slug": "test-cant-override-slug-on-creation", "details": "this is just a test project", "description": "the project is being tested", "minimum_age": 18, "address": {"typed_address": "r. tecainda, 81, sao paulo"}, "disponibility": {"type": "work", "work": {"description": "abc"}}}

@override_settings(OVP_PROJECTS={"CAN_CREATE_PROJECTS_WITHOUT_ORGANIZATION": True})
class ProjectResourceViewSetTestCase(TestCase):
  def test_cant_create_project_unauthenticated(self):
    """Assert that it's not possible to create a project while unauthenticated"""
    client = APIClient()
    response = client.post(reverse("project-list"), {}, format="json")

    self.assertTrue(response.data["detail"] == "Authentication credentials were not provided.")
    self.assertTrue(response.status_code == 401)

  def test_can_create_project(self):
    """Assert that it's possible to create a project while authenticated"""
    user = User.objects.create_user(email="test_can_create_project@gmail.com", password="testcancreate")

    data = copy.copy(base_project)

    client = APIClient()
    client.force_authenticate(user=user)

    response = client.post(reverse("project-list"), data, format="json")

    self.assertTrue(response.status_code == 201)
    self.assertTrue(response.data["id"])
    self.assertTrue(response.data["name"] == data["name"])
    self.assertTrue(response.data["slug"] == "test-project")
    self.assertTrue(response.data["details"] == data["details"])
    self.assertTrue(response.data["description"] == data["description"])
    self.assertTrue(response.data["minimum_age"] == data["minimum_age"])

    project = Project.objects.get(pk=response.data["id"])
    self.assertTrue(project.owner.id == user.id)
    self.assertTrue(project.address.typed_address == data["address"]["typed_address"])


  def test_cant_create_project_empty_name(self):
    """Assert that it's not possible to create a project with empty name"""
    user = User.objects.create_user(email="test_can_create_project@gmail.com", password="testcancreate")

    client = APIClient()
    client.force_authenticate(user=user)

    data = copy.copy(base_project)
    data["name"] = ""

    response = client.post(reverse("project-list"), data, format="json")
    self.assertTrue(response.data["name"][0] == "This field may not be blank.")


  def test_project_retrieval(self):
    """Assert projects can be retrieved"""
    user = User.objects.create_user(email="test_retrieval@gmail.com", password="testretrieval")

    client = APIClient()
    client.force_authenticate(user=user)

    data = copy.copy(base_project)
    response = client.post(reverse("project-list"), data, format="json")

    response = client.get(reverse("project-detail", ["test-project"]), format="json")

    self.assertTrue(response.data["name"] == data["name"])
    self.assertTrue(response.data["slug"] == "test-project")
    self.assertTrue(response.data["details"] == data["details"])
    self.assertTrue(response.data["description"] == data["description"])
    self.assertTrue(response.data["published"] == False)
    self.assertTrue(type(response.data["owner"]) in [dict, OrderedDict])
    self.assertTrue(type(response.data["applies"]) is list)
    self.assertTrue(type(response.data["applied_count"]) is int)
    self.assertTrue(type(response.data["max_applies_from_roles"]) is int)



@override_settings(OVP_PROJECTS={"CAN_CREATE_PROJECTS_WITHOUT_ORGANIZATION": True})
class ProjectCloseTestCase(TestCase):
  def setUp(self):
    user = User.objects.create_user(email="test_close@gmail.com", password="testclose")
    self.client = APIClient()
    self.client.force_authenticate(user=user)

    data = copy.copy(base_project)
    self.project = self.client.post(reverse("project-list"), data, format="json")

  def test_cant_close_project_if_not_owner_or_organization_member(self):
    """ Assert that it's not possible to close a project if not the owner or organization member """
    user = User.objects.create_user(email="otheruser@gmail.com", password="otheruser")
    self.client.force_authenticate(user=user)
    response = self.client.post(reverse("project-close", ["test-project"]), format="json")
    self.assertTrue(response.status_code == 403)


  def test_can_close_project(self):
    """ Assert that it's possible to close a project """
    response = self.client.post(reverse("project-close", ["test-project"]), format="json")
    self.assertTrue(response.status_code == 200)
    self.assertTrue(response.data["closed"] == True)
    self.assertTrue(response.data["closed"])


@override_settings(OVP_PROJECTS={"CAN_CREATE_PROJECTS_WITHOUT_ORGANIZATION": False})
class ProjectWithOrganizationTestCase(TestCase):
  def setUp(self):
    self.user = User.objects.create_user(email="test_can_create_project@gmail.com", password="testcancreate")
    self.another_user = User.objects.create_user(email="test_another_user@test.com", password="testcancreate")
    self.data = copy.copy(base_project)
    self.client = APIClient()
    self.client.force_authenticate(user=self.user)

  def test_no_organization(self):
    """Test no organization returns error"""
    response = self.client.post(reverse("project-list"), self.data, format="json")
    self.assertTrue(response.data["organization"] == ["This field is required."])
    self.assertTrue(response.status_code == 400)

  def test_organization_is_int(self):
    """Test organization field must be int"""
    self.data['organization'] = 'str'

    response = self.client.post(reverse("project-list"), self.data, format="json")
    self.assertTrue(response.data["organization"] == ["Incorrect type. Expected pk value, received str."])
    self.assertTrue(response.status_code == 400)

  def test_user_is_owner_or_member(self):
    """Test user is owner or member of organization"""
    wrong_org = Organization(name="test", type=0, owner=self.another_user)
    wrong_org.save()

    self.data['organization'] = wrong_org.pk
    response = self.client.post(reverse("project-list"), self.data, format="json")
    self.assertTrue(response.status_code == 403)

  @override_settings(OVP_PROJECTS={"CAN_CREATE_PROJECTS_IN_ANY_ORGANIZATION": True})
  def test_can_create_in_any_organization_if_settings_allow(self):
    """Test user can create project inside any organization if properly configured"""
    wrong_org = Organization(name="test", type=0, owner=self.another_user)
    wrong_org.save()

    self.data['organization'] = wrong_org.pk
    response = self.client.post(reverse("project-list"), self.data, format="json")
    self.assertTrue(response.status_code == 201)

  def test_can_create(self):
    """Test user can create project with valid organization"""
    org = Organization(name="test", type=0, owner=self.user)
    org.save()
    org.members.add(self.another_user)

    self.data['organization'] = org.pk
    response = self.client.post(reverse("project-list"), self.data, format="json")
    self.assertTrue(response.status_code == 201)

    self.client.force_authenticate(self.another_user)
    response = self.client.post(reverse("project-list"), self.data, format="json")
    self.assertTrue(response.status_code == 201)


class ManageableProjectsRouteTestCase(TestCase):
  def setUp(self):
    self.user = User.objects.create_user(email="test_can_create_project@gmail.com", password="testcancreate")
    self.user.save()
    self.user2 = User.objects.create_user(email="test_can_create_project2@gmail.com", password="testcancreate")
    self.user2.save()
    self.organization = Organization(name="test", type=0, owner=self.user)
    self.organization.save()
    self.organization2 = Organization(name="test2", type=0, owner=self.user2)
    self.organization2.save()
    self.organization2.members.add(self.user)

    p = Project(name="test project 1", owner=self.user)
    p.save()

    p = Project(name="test project 2", owner=self.user, organization=self.organization)
    p.save()

    p = Project(name="test project 3", owner=self.user2, organization=self.organization2)
    p.save()

    self.client = APIClient()
    self.client.force_authenticate(user=self.user)

  def test_requires_authentication(self):
    """Test hitting route unauthenticated returns 401"""
    client = APIClient()
    response = client.get(reverse("project-manageable"), {}, format="json")
    self.assertTrue(response.status_code == 401)


  def test_returns_projects(self):
    """Test hitting route authenticated returns projects"""
    response = self.client.get(reverse("project-manageable"), {}, format="json")
    self.assertTrue(response.status_code == 200)
    self.assertTrue(len(response.data) == 3)


@override_settings(OVP_PROJECTS={"CAN_CREATE_PROJECTS_WITHOUT_ORGANIZATION": True})
class ProjectResourceUpdateTestCase(TestCase):
  def setUp(self):
    self.user = User.objects.create_user(email="test_can_create_project@gmail.com", password="testcancreate")
    self.data = copy.copy(base_project)
    self.client = APIClient()
    self.client.force_authenticate(user=self.user)

    response = self.client.post(reverse("project-list"), self.data, format="json")
    self.assertTrue(response.status_code == 201)

  def test_wrong_user_cant_update(self):
    """Test only owner can update project"""
    wrong_user = User.objects.create_user(email="wrong_user@gmail.com", password="testcancreate")
    wrong_user.save()
    self.client.force_authenticate(user=wrong_user)

    response = self.client.patch(reverse("project-detail", ["test-project"]), {}, format="json")
    self.assertTrue(response.status_code == 403)

  def test_update_fields(self):
    """Test patch request update fields"""
    updated_project = {"name": "test update", "details": "update", "description": "update"}
    response = self.client.patch(reverse("project-detail", ["test-project"]), updated_project, format="json")
    self.assertTrue(response.status_code == 200)
    self.assertTrue(response.data["name"] == "test update")
    self.assertTrue(response.data["details"] == "update")
    self.assertTrue(response.data["description"] == "update")

    user = User.objects.create_user(email="another@user.com", password="testcancreate")
    organization = Organization(name="test", type=0, owner=self.user)
    organization.save()
    organization.members.add(user)
    project = Project.objects.get(pk=response.data['id'])
    project.organization = organization
    project.save()
    self.client.force_authenticate(user)
    response = self.client.patch(reverse("project-detail", ["test-project"]), updated_project, format="json")
    self.assertTrue(response.status_code == 200)

  def test_update_address(self):
    """Test patch request update address resource"""
    updated_project = {"address": {"typed_address": "r. capote valente, 701, sao paulo"}}
    response = self.client.patch(reverse("project-detail", ["test-project"]), updated_project, format="json")

    self.assertTrue(response.status_code == 200)
    self.assertTrue(response.data["address"]["typed_address"] == "r. capote valente, 701, sao paulo")

  def test_update_disponibility(self):
    """Test patch request update disponibility resource"""
    updated_project = {"disponibility": {"type": "job", "job": {"dates": [{"name": "update", "start_date": "2013-01-29T12:34:56.123Z", "end_date": "2013-01-29T13:34:56.123Z"}, {"name": "test1", "start_date": "2013-02-01T12:34:56.123Z", "end_date": "2013-02-01T13:34:56.123Z"}]}}}
    response = self.client.patch(reverse("project-detail", ["test-project"]), updated_project, format="json")

    self.assertTrue(response.status_code == 200)
    self.assertTrue(response.data["disponibility"]["type"] == "job")
    self.assertTrue(response.data["disponibility"]["job"]["dates"][0]["name"] == "update")


    updated_project = {"disponibility": {"type": "work", "work": {"description": "update"}}}
    response = self.client.patch(reverse("project-detail", ["test-project"]), updated_project, format="json")

    self.assertTrue(response.status_code == 200)
    self.assertTrue(response.data["disponibility"]["type"] == "work")
    self.assertTrue(response.data["disponibility"]["work"]["description"] == "update")

  def test_update_roles(self):
    """Test patch request update roles resource"""
    updated_project = {"roles": [{"name": "test", "prerequisites": "test2", "details": "test3", "vacancies": 5}]}
    response = self.client.patch(reverse("project-detail", ["test-project"]), updated_project, format="json")

    self.assertTrue(response.status_code == 200)
    self.assertTrue(response.data["roles"] == updated_project["roles"])


@override_settings(OVP_PROJECTS={"CAN_CREATE_PROJECTS_WITHOUT_ORGANIZATION": True})
class DisponibilityTestCase(TestCase):
  def setUp(self):
    self.user = User.objects.create_user(email="test_can_create_project@gmail.com", password="testcancreate")
    self.data = copy.copy(base_project)
    self.client = APIClient()
    self.client.force_authenticate(user=self.user)

  def test_no_disponibility(self):
    """Test no disponibility returns error"""
    del self.data["disponibility"]
    response = self.client.post(reverse("project-list"), self.data, format="json")
    self.assertTrue(response.data["disponibility"] == ["This field is required."])
    self.assertTrue(response.status_code == 400)

  def test_disponibility_type_required(self):
    """Test disponibility type is required"""
    self.data["disponibility"] = {}
    response = self.client.post(reverse("project-list"), self.data, format="json")
    self.assertTrue(response.data["disponibility"]["type"] == ["This field is required."])
    self.assertTrue(response.status_code == 400)

  def test_type_not_work_or_job(self):
    """Test disponibility type can't be different than work or job"""
    self.data["disponibility"] = {"type": "test"}
    response = self.client.post(reverse("project-list"), self.data, format="json")
    self.assertTrue(response.data["disponibility"]["type"] == ["Must have either be 'work' or 'job'."])
    self.assertTrue(response.status_code == 400)

  def test_empty_job_or_work(self):
    """Test empty job or work returns error"""
    self.data["disponibility"] = {"type": "job"}
    response = self.client.post(reverse("project-list"), self.data, format="json")
    self.assertTrue(response.data["disponibility"]["job"] == ["This field is required if type=\"job\"."])
    self.assertTrue(response.status_code == 400)

    self.data["disponibility"] = {"type": "work"}
    response = self.client.post(reverse("project-list"), self.data, format="json")
    self.assertTrue(response.data["disponibility"]["work"] == ["This field is required if type=\"work\"."])
    self.assertTrue(response.status_code == 400)

  def test_work_description_required(self):
    """Test work description is required"""
    self.data["disponibility"] = {"type": "work", "work": {}}
    response = self.client.post(reverse("project-list"), self.data, format="json")
    self.assertTrue(response.data["disponibility"]["work"]["description"] == ["This field is required."])
    self.assertTrue(response.status_code == 400)

  def test_correct_work(self):
    """Test correct work returns success"""
    self.data["disponibility"] = {"type": "work", "work": {"description": "test desc", "weekly_hours": 6}}
    response = self.client.post(reverse("project-list"), self.data, format="json")
    self.assertTrue(response.data["disponibility"]["type"] == "work")
    self.assertTrue(response.data["disponibility"]["work"]["description"] == "test desc")
    self.assertTrue(response.data["disponibility"]["work"]["weekly_hours"] == 6)
    self.assertTrue(response.status_code == 201)

  def test_job_dates_required(self):
    """Test job dates is required"""
    self.data["disponibility"] = {"type": "job", "job": {}}
    response = self.client.post(reverse("project-list"), self.data, format="json")
    self.assertTrue(response.data["disponibility"]["job"]["dates"] == ["This field is required."])
    self.assertTrue(response.status_code == 400)

  def test_job_dates_cant_be_empty(self):
    """Test job dates can't be empty"""
    self.data["disponibility"] = {"type": "job", "job": {"dates": []}}
    response = self.client.post(reverse("project-list"), self.data, format="json")
    self.assertTrue(response.data["disponibility"]["job"]["dates"] == ["Must have at least one date."])
    self.assertTrue(response.status_code == 400)

  def test_job_dates_cant_be_wrong_type(self):
    """Test job dates can't be wrong type"""
    self.data["disponibility"] = {"type": "job", "job": {"dates": ''}}
    response = self.client.post(reverse("project-list"), self.data, format="json")
    self.assertTrue(response.data["disponibility"]["job"]["dates"]["non_field_errors"] == ["Expected a list of items but got type \"str\"."])
    self.assertTrue(response.status_code == 400)

  def test_job_dates_cant_have_bad_formatted_date(self):
    """Test job dates can't have bad formatted date"""
    self.data["disponibility"] = {"type": "job", "job": {"dates": [{"start_date": "abc", "end_date": "abc"}]}}
    response = self.client.post(reverse("project-list"), self.data, format="json")
    self.assertTrue(response.data["disponibility"]["job"]["dates"][0]["start_date"] == ["Datetime has wrong format. Use one of these formats instead: YYYY-MM-DDThh:mm[:ss[.uuuuuu]][+HH:MM|-HH:MM|Z]."])
    self.assertTrue(response.data["disponibility"]["job"]["dates"][0]["end_date"] == ["Datetime has wrong format. Use one of these formats instead: YYYY-MM-DDThh:mm[:ss[.uuuuuu]][+HH:MM|-HH:MM|Z]."])
    self.assertTrue(response.status_code == 400)

  def test_job_returns_success(self):
    """Test correct job returns success"""
    self.data["disponibility"] = {"type": "job", "job": {"dates": [{"name": "test1", "start_date": "2013-01-29T12:34:56.123Z", "end_date": "2013-01-29T13:34:56.123Z"}, {"name": "test1", "start_date": "2013-02-01T12:34:56.123Z", "end_date": "2013-02-01T13:34:56.123Z"}]}}
    response = self.client.post(reverse("project-list"), self.data, format="json")
    self.assertTrue(response.status_code == 201)
    self.assertTrue(response.data["disponibility"]["job"]["dates"][0]["start_date"] == "2013-01-29T12:34:56.123000Z")
    self.assertTrue(response.data["disponibility"]["job"]["dates"][0]["end_date"] == "2013-01-29T13:34:56.123000Z")
    self.assertTrue(response.data["disponibility"]["job"]["dates"][1]["start_date"] == "2013-02-01T12:34:56.123000Z")
    self.assertTrue(response.data["disponibility"]["job"]["dates"][1]["end_date"] == "2013-02-01T13:34:56.123000Z")

    self.assertTrue(response.data["disponibility"]["job"]["start_date"] == "2013-01-29T12:34:56.123000Z")
    self.assertTrue(response.data["disponibility"]["job"]["end_date"] == "2013-02-01T13:34:56.123000Z")



@override_settings(OVP_PROJECTS={"CAN_CREATE_PROJECTS_WITHOUT_ORGANIZATION": True})
class VolunteerRoleTestCase(TestCase):
  def setUp(self):
    self.user = User.objects.create_user(email="test_can_create_project@gmail.com", password="testcancreate")
    self.data = copy.copy(base_project)
    self.client = APIClient()
    self.client.force_authenticate(user=self.user)


  def test_roles_is_correct_type(self):
    """Test roles is correct type"""
    self.data["roles"] = {"name": "test", "prerequisites": "test2", "details": "test3", "vacancies": 5}
    response = self.client.post(reverse("project-list"), self.data, format="json")
    self.assertTrue(response.data["roles"]["non_field_errors"] == ["Expected a list of items but got type \"dict\"."])
    self.assertTrue(response.status_code == 400)


  def test_roles_get_saved(self):
    """Test roles get saved"""
    self.data["roles"] = [{"name": "test", "prerequisites": "test2", "details": "test3", "vacancies": 5}]
    response = self.client.post(reverse("project-list"), self.data, format="json")
    self.assertTrue(response.status_code == 201)
    self.assertTrue(response.data["roles"] == self.data["roles"])

    response = self.client.get(reverse("project-detail", ['test-project']), format="json")
    self.assertTrue(response.status_code == 200)
    self.assertTrue(response.data["roles"] == self.data["roles"])



class ApplyTestCase(TestCase):
  def test_can_apply_to_project(self):
    """Assert that authenticated user can apply to project"""
    owner = User.objects.create_user(email="owner_user@gmail.com", password="test_owner")
    project = Project(name="test project", details="abc", description="abc", owner=owner)
    project.save()

    user = User.objects.create_user(email="apply_user@gmail.com", password="apply_user")

    client = APIClient()
    client.force_authenticate(user=user)

    response = client.post(reverse("project-apply", ["test-project"]), format="json")
    self.assertTrue(response.data["detail"] == "Successfully applied.")
    self.assertTrue(response.status_code == 200)

    response = client.post(reverse("project-apply", ["test-project"]), format="json")
    self.assertTrue(response.data["non_field_errors"][0] == "The fields email, project must make a unique set.")

    response = client.get(reverse("project-detail", ["test-project"]), format="json")
    self.assertTrue(type(response.data["applies"][0]["user"]) in [dict, OrderedDict])


  def test_can_reapply_to_project(self):
    """Assert that user can reapply to a project"""
    user = User.objects.create_user(email="owner_user@gmail.com", password="test_owner")
    project = Project(name="test project", details="abc", description="abc", owner=user)
    project.save()

    user = User.objects.create_user(email="apply_user@gmail.com", password="apply_user")

    client = APIClient()
    client.force_authenticate(user=user)

    # Apply
    response = client.post(reverse("project-apply", ["test-project"]), format="json")
    self.assertTrue(response.data["detail"] == "Successfully applied.")
    self.assertTrue(response.status_code == 200)

    project = Project.objects.get(slug="test-project")
    self.assertTrue(project.applied_count == 1)

    # Unapply
    response = client.post(reverse("project-unapply", ["test-project"]), format="json")
    self.assertTrue(response.data["detail"] == "Successfully unapplied.")

    a = Apply.objects.last()
    self.assertTrue(a.canceled == True)
    self.assertTrue(a.canceled_date)

    project = Project.objects.get(slug="test-project")
    self.assertTrue(project.applied_count == 0)

    # Reapply
    response = client.post(reverse("project-apply", ["test-project"]), format="json")
    self.assertTrue(response.data["detail"] == "Successfully applied.")
    self.assertTrue(response.status_code == 200)

    project = Project.objects.get(slug="test-project")
    self.assertTrue(project.applied_count == 1)

    a = Apply.objects.last()
    self.assertTrue(a.canceled == False)
    self.assertTrue(a.canceled_date == None)

  def test_cant_unapply_if_not_apply_or_unauthenticated(self):
    """Assert that user can't unapply if not already applied or unauthenticated"""
    user = User.objects.create_user(email="owner_user@gmail.com", password="test_owner")
    project = Project(name="test project", details="abc", description="abc", owner=user)
    project.save()

    user = User.objects.create_user(email="apply_user@gmail.com", password="apply_user")

    client = APIClient()

    response = client.post(reverse("project-unapply", ["test-project"]), format="json")
    self.assertTrue(response.data["detail"] == "Authentication credentials were not provided.")
    self.assertTrue(response.status_code == 401)

    client.force_authenticate(user=user)
    response = client.post(reverse("project-unapply", ["test-project"]), format="json")
    self.assertTrue(response.data["detail"] == "This is user is not applied to this project.")
    self.assertTrue(response.status_code == 400)

  def test_cant_apply_to_inexistent_project(self):
    """Assert that user can't apply to inexistent project"""
    user = User.objects.create_user(email="apply_user@gmail.com", password="apply_user")

    client = APIClient()
    client.force_authenticate(user=user)

    response = client.post(reverse("project-apply", ["test-project"]), format="json")
    self.assertTrue(response.data["detail"] == "Not found.")
    self.assertTrue(response.status_code == 404)


  @override_settings(OVP_PROJECTS={"UNAUTHENTICATED_APPLY": False})
  def test_unauthenticated_user_cant_apply_to_project(self):
    """Assert that unauthenticated user cant apply to project"""
    user = User.objects.create_user(email="owner_user@gmail.com", password="test_owner")
    project = Project(name="test project", details="abc", description="abc", owner=user)
    project.save()

    client = APIClient()

    response = client.post(reverse("project-apply", ["test-project"]), {"email": "testemail@test.com"}, format="json")
    self.assertTrue(response.data["detail"] == "Authentication credentials were not provided.")
    self.assertTrue(response.status_code == 401)


  @override_settings(OVP_PROJECTS={"UNAUTHENTICATED_APPLY": True})
  def test_unauthenticated_user_can_apply_to_project(self):
    """Assert that unauthenticated user can apply to project if properly configured"""
    user = User.objects.create_user(email="owner_user@gmail.com", password="test_owner")
    project = Project(name="test project", details="abc", description="abc", owner=user)
    project.save()

    client = APIClient()

    response = client.post(reverse("project-apply", ["test-project"]), {"email": "testemail@test.com"}, format="json")
    self.assertTrue(response.data["detail"] == "Successfully applied.")
    self.assertTrue(response.status_code == 200)


  def test_can_read_applies(self):
    """Assert that organization member can view project applies"""
    owner = User.objects.create_user(email="owner_user@gmail.com", password="test_owner")
    project = Project(name="test project", details="abc", description="abc", owner=owner)
    project.save()

    user = User.objects.create_user(email="apply_user@gmail.com", password="apply_user")

    client = APIClient()
    client.force_authenticate(user=user)
    response = client.post(reverse("project-apply", ["test-project"]), format="json")
    self.assertTrue(response.data["detail"] == "Successfully applied.")
    self.assertTrue(response.status_code == 200)


    client = APIClient()
    client.force_authenticate(user=owner)
    response = client.get(reverse("project-applies", ["test-project"]), format="json")
    self.assertTrue(response.status_code == 200)
    self.assertTrue("email" in response.data[0])
    self.assertTrue("date" in response.data[0])
    self.assertTrue("canceled" in response.data[0])
    self.assertTrue("canceled_date" in response.data[0])
    self.assertTrue("status" in response.data[0])
    self.assertTrue("name" in response.data[0]["user"])
    self.assertTrue("avatar" in response.data[0]["user"])
    self.assertTrue("email" in response.data[0]["user"])
    self.assertTrue("phone" in response.data[0]["user"])

    url = reverse("project-applies", ["test-project", "csv"])
    response = client.get(url, format="csv")
    self.assertTrue(response.status_code == 200)

    client = APIClient()
    client.force_authenticate(user=user)
    response = client.get(reverse("project-applies", ["test-project"]), format="json")
    self.assertTrue(response.status_code == 403)



import sys
from io import StringIO

from django.test import TestCase
from django.utils import timezone

from ovp_users.models import User
from ovp_projects.models import Project, Job
from ovp_projects.management.commands.close_finished_projects import Command as CloseFinishedProjects



class TestCloseProjectsCommand(TestCase):
  def test_close_finished_projects(self):
    """Test close_finished_projects command"""
    user = User.objects.create_user(email="test_owner@test.com", password="test_owner")
    user.save()

    p = Project(name="test", owner=user)
    p.save()

    job = Job(start_date=timezone.now(), end_date=timezone.now(), project=p)
    job.save()

    saved_stdout = sys.stdout
    try:
      out = StringIO()
      sys.stdout = out

      cmd = CloseFinishedProjects()
      cmd.handle()

      output = out.getvalue().strip()
    finally:
      sys.stdout = saved_stdout

    p = Project.objects.get(pk=p.pk)
    self.assertTrue(output == "Closing 1 finished projects")
    self.assertTrue(p.closed)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-11-28 19:01
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ovp_projects', '0019_auto_20161121_0140'),
    ]

    operations = [
        migrations.AddField(
            model_name='work',
            name='can_be_done_remotely',
            field=models.BooleanField(default=False, verbose_name='This job can be done remotely'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-11-16 18:09
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ovp_projects', '0015_auto_20161116_0141'),
    ]

    operations = [
        migrations.AlterField(
            model_name='project',
            name='slug',
            field=models.SlugField(blank=True, max_length=100, null=True, unique=True),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-11-21 01:40
from __future__ import unicode_literals

from django.db import migrations, models
from django.utils import timezone


class Migration(migrations.Migration):

    dependencies = [
        ('ovp_projects', '0018_auto_20161118_1214'),
    ]

    operations = [
        migrations.AlterField(
            model_name='jobdate',
            name='end_date',
            field=models.DateTimeField(default=timezone.now(), verbose_name='End date'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='jobdate',
            name='start_date',
            field=models.DateTimeField(default=timezone.now(), verbose_name='Start date'),
            preserve_default=False,
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-10-28 19:36
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('ovp_organizations', '0003_organization_type'),
        ('ovp_projects', '0007_project_image'),
    ]

    operations = [
        migrations.AddField(
            model_name='project',
            name='organization',
            field=models.ForeignKey(default=None, on_delete=django.db.models.deletion.CASCADE, to='ovp_organizations.Organization'),
            preserve_default=False,
        ),
    ]



from django.test import TestCase
from django.utils import timezone

from ovp_projects.models import Project
from ovp_projects.models import VolunteerRole
from ovp_projects.models import Work
from ovp_projects.models import Job
from ovp_users.models import User

class ProjectModelTestCase(TestCase):
  def test_project_return_owner_email_and_phone(self):
    """ Assert the methods .get_phone() and .get_email() return owner phone and email address """
    phone = "123456789"
    email = "test_returned@test.com"

    user = User.objects.create_user(email=email, password="test_returned")
    user.phone = phone
    user.save()

    project = Project(name="test project", slug="test slug", details="abc", description="abc", owner=user)
    project.save()

    self.assertTrue(project.get_phone() == phone)
    self.assertTrue(project.get_email() == email)

  def test_project_delete(self):
    """ Assert project delete method modifies .deleted and .published columns """
    user = User.objects.create_user(email="test_deleted@deleted.com", password="test_deleted")
    project = Project(name="test project", slug="test deleted", details="abc", description="abc", owner=user)
    project.published = True
    project.save()

    self.assertTrue(project.deleted == False)
    self.assertTrue(project.published == True)
    self.assertTrue(project.deleted_date == None)

    project.delete()

    self.assertTrue(project.deleted == True)
    self.assertTrue(project.published == False)
    self.assertTrue(project.deleted_date)

  def test_project_publish(self):
    """ Assert publishing a project updates .published_date """
    user = User.objects.create_user(email="test_published@date.com", password="test_published_date")
    project = Project(name="test project", slug="test published date", details="abc", description="abc", owner=user)
    project.save()
    self.assertTrue(project.published_date == None)

    project.published = True
    project.save()

    self.assertTrue(project.published_date)

  def test_project_close(self):
    """ Assert closing a project updates .closed_date """
    user = User.objects.create_user(email="test_close@date.com", password="test_close_date")
    project = Project(name="test project", slug="test close date", details="abc", description="abc", owner=user)
    project.save()
    self.assertTrue(project.closed_date == None)

    project.closed = True
    project.save()

    self.assertTrue(project.closed_date)

  def test_excerpt_from_details(self):
    """ Assert that if a project has no .description, it will use 100 chars from .details as an excerpt """
    details = ("a" * 100) + "b"
    expected_description = "a" * 100

    user = User.objects.create_user(email="test_excerpt@text.com", password="test_excerpt_text")
    project = Project(name="test project", slug="test excerpt text", details=details, owner=user)
    project.save()

    project = Project.objects.get(pk=project.id)
    self.assertTrue(project.description == expected_description)

  def test_str_method_returns_name(self):
    """ Assert that .__str__() method returns project name """
    user = User.objects.create_user(email="test_str@test.com", password="test_str_test")
    project = Project(name="test str", slug="test str test", details="abc", owner=user)
    project.save()

    self.assertTrue(project.__str__() == "test str")

  def test_slug_generation_on_create(self):
    """ Assert that slug is generated on create """
    user = User.objects.create_user(email="test_str@test.com", password="test_str_test")
    project = Project(name="test slug", slug="another-slug", details="abc", owner=user)
    project.save()

    self.assertTrue(project.slug == "test-slug")

  def test_slug_doesnt_repeat(self):
    """ Assert that slug does not repeat """
    user = User.objects.create_user(email="test_str@test.com", password="test_str_test")
    project = Project(name="test slug", details="abc", owner=user)
    project.save()
    self.assertTrue(project.slug == "test-slug")

    project = Project(name="test slug", details="abc", owner=user)
    project.save()
    self.assertTrue(project.slug == "test-slug-1")

  def test_slug_is_not_generated_without_name(self):
    """ Assert that slug is not generated without name """
    user = User.objects.create_user(email="test_slug@test.com", password="test_slug_test")
    project = Project(details="abc", owner=user)
    self.assertTrue(project.generate_slug() == None)



class VolunteerRoleModelTestCase(TestCase):
  def test_str_method_returns_role_info(self):
    """ Assert that VolunteerRole.__str__() method returns role info """
    role = VolunteerRole(name="test role", details="a", prerequisites="b", vacancies=5)
    role.save()

    self.assertTrue(role.__str__() == "test role - a - b (5 vacancies)")


class WorkModelTestCase(TestCase):
  def test_str_method_returns_work_info(self):
    """ Assert that Work.__str__() method returns hours per week """
    work = Work(weekly_hours=4, description="abc")
    work.save()

    self.assertTrue(work.__str__() == "4 hours per week")


class JobModelTestCase(TestCase):
  def test_str_method_returns_job_info(self):
    """ Assert that Job.__str__() method returns .start_date and .end_date """
    user = User.objects.create_user(email="test@email.com", password="test_email")
    user.save()
    project = Project(name="test project", slug="test slug", details="abc", description="abc", owner=user)
    project.save()

    start = timezone.now()
    end = timezone.now()
    job = Job(start_date=start, end_date=end, project=project)
    job.save()

    human_time = lambda x: x.strftime("%d/%m/%Y")

    self.assertTrue(job.__str__() == "{}: {} ~ {}".format(project.name, human_time(start), human_time(end)))



from django.contrib import admin
from django import forms
from django.utils.translation import ugettext_lazy as _

from ovp_projects.models import Apply

from ovp_core.mixins import CountryFilterMixin


class ApplyAdmin(admin.ModelAdmin, CountryFilterMixin):
  fields = [
    ('id', 'project__name', 'status'),
    'user', 'project', 'project__organization__name',
    ('canceled', 'canceled_date', 'date'),
    'email'
    ]

  list_display = [
    'id', 'date', 'user__name', 'user__email', 'user__phone', 'project__name',
    'project__organization__name', 'project__address', 'status', 'canceled'
    ]

  list_filter = ['date', 'canceled',]

  list_editable = ['canceled',]

  search_fields = [
    'user__name', 'user__email', 'project__name',
    'project__organization__name'
    ]

  readonly_fields = [
    'id', 'project__name', 'user', 'project__organization__name',
    ]

  raw_id_fields = []

  def user__name(self, obj):
    if obj.user:
      return obj.user.name
    else:
      return _('None')

  user__name.short_description = _('Name')
  user__name.admin_order_field = 'user__name'

  def user__email(self, obj):
    if obj.user:
      return obj.user.email
    else:
      return _('None')
  user__email.short_description = _('E-mail')
  user__email.admin_order_field = 'user__email'

  def user__phone(self, obj):
    if obj.user:
      return obj.user.phone
    else:
      return _('None')
  user__phone.short_description = _('Phone')
  user__phone.admin_order_field = 'user__phone'

  def project__name(self, obj):
    if obj.project:
      return obj.project.name
    else:
      return _('None')
  project__name.short_description = _('Project')
  project__name.admin_order_field = 'project__name'

  def project__organization__name(self, obj):
    if obj.project and obj.project.organization:
      return obj.project.organization.name
    else:
      return _('None')
  project__organization__name.short_description = _('Organization')
  project__organization__name.project__organization__name = 'project__organization__name'

  def project__address(self, obj):
    if obj.project:
      return obj.project.address
    else:
      return _('None')
  project__address.short_description = _('Address')
  project__address.admin_order_field = 'project__address'

  def get_queryset(self, request):
    qs = super(ApplyAdmin, self).get_queryset(request)
    return self.filter_by_country(request, qs, 'project__address')

admin.site.register(Apply, ApplyAdmin)





# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

setup(
    name='ovp-projects',
    version='1.1.8',
    author=u'Atados',
    author_email='arroyo@atados.com.br',
    packages=find_packages(),
    include_package_data=True,
    url='https://github.com/OpenVolunteeringPlatform/django-ovp-projects',
    download_url = 'https://github.com/OpenVolunteeringPlatform/django-ovp-projects/tarball/1.1.8',
    license='AGPL',
    description='This module has core functionality for' + \
                ' ovp projects, such as creation, editing' + \
                ' and retrieving.',
    long_description=open('README.rst', encoding='utf-8').read(),
    zip_safe=False,
    install_requires = [
      'Django>=1.10.1,<1.11.0',
      'djangorestframework>=3.5.0,<3.6.0',
      'djangorestframework-jwt>=1.8.0,<1.9.0',
      'python-dateutil>=2.5.3,<2.6.0',
      'codecov>=2.0.5,<2.1.0',
      'coverage>=4.2,<4.3.0',
      'ovp-users>=1.0.0,<2.0.0',
      'ovp-core>=1.0.0,<2.0.0',
      'ovp-uploads>=1.0.0,<2.0.0',
      'ovp-organizations>=1.0.0,<2.0.0',
    ]
)



from django.contrib import admin
from django import forms
from django.utils.translation import ugettext_lazy as _

from ovp_projects.models import Project, VolunteerRole
from .job import JobInline
from .work import WorkInline

from ovp_core.mixins import CountryFilterMixin


class VolunteerRoleInline(admin.TabularInline):
  model = VolunteerRole


class ProjectAdmin(admin.ModelAdmin, CountryFilterMixin):
  fields = [
    ('id', 'highlighted'), ('name', 'slug'),
    ('organization', 'owner'),

    ('owner__name', 'owner__email', 'owner__phone'),

    ('applied_count', 'max_applies_from_roles'),

    ('can_be_done_remotely'),

    ('published', 'closed', 'deleted'),
    ('published_date', 'closed_date', 'deleted_date'),

    'address',
    'image',

    ('created_date', 'modified_date'),

    'description', 'details',
    'skills', 'causes',
    ]

  list_display = [
    'id', 'created_date', 'name', 'organization__name', 'applied_count', # fix: CIDADE, PONTUAL OU RECORRENTE
    'highlighted', 'published', 'closed', 'deleted', #fix: EMAIL STATUS
    ]

  list_filter = [
    'created_date', # fix: PONTUAL OU RECORRENTE
    'highlighted', 'published', 'closed', 'deleted'
  ]

  list_editable = [
    'highlighted', 'published', 'closed'
  ]

  search_fields = [
    'name', 'organization__name'
  ]

  readonly_fields = [
    'id', 'created_date', 'modified_date', 'published_date', 'closed_date', 'deleted_date', 'applied_count', 'max_applies_from_roles',
    'owner__name', 'owner__email', 'owner__phone',
    'can_be_done_remotely'
  ]

  raw_id_fields = []

  filter_horizontal = ('skills', 'causes',)

  inlines = [
    VolunteerRoleInline,
    JobInline, WorkInline
  ]

  def can_be_done_remotely(self, obj):
    if obj.hasattr('job') and obj.job:
      return obj.job.can_be_done_remotely
    elif obj.hasattr('work') and obj.work:
      return obj.job.can_be_done_remotely
    else:
      return _('Type not specified')
  can_be_done_remotely.short_description = _('Can be done remotely?')

  def organization__name(self, obj):
    if obj.organization:
      return obj.organization.name
    else:
      return _('None')
  organization__name.short_description = _('Organization')
  organization__name.admin_order_field = 'organization__name'

  def owner__name(self, obj):
    return obj.owner and obj.owner.name or _('Owner not assigned')
  owner__name.short_description = _('Owner name')
  owner__name.admin_order_field = 'owner__name'

  def owner__email(self, obj):
    return obj.owner and obj.owner.email or _('Owner not assigned')
  owner__email.short_description = _('Owner email')
  owner__email.admin_order_field = 'owner__email'

  def owner__phone(self, obj):
    return obj.owner and obj.owner.phone or _('Owner not assigned')
  owner__phone.short_description = _('Owner phone')
  owner__phone.admin_order_field = 'owner__phone'

  def get_queryset(self, request):
    qs = super(ProjectAdmin, self).get_queryset(request)
    return self.filter_by_country(request, qs, 'address')


admin.site.register(Project, ProjectAdmin)





# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2017-01-12 14:17
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('ovp_projects', '0022_auto_20161220_1914'),
    ]

    operations = [
        migrations.AlterField(
            model_name='apply',
            name='canceled',
            field=models.BooleanField(default=False, verbose_name='canceled'),
        ),
        migrations.AlterField(
            model_name='apply',
            name='canceled_date',
            field=models.DateTimeField(blank=True, null=True, verbose_name='canceled date'),
        ),
        migrations.AlterField(
            model_name='apply',
            name='date',
            field=models.DateTimeField(auto_now_add=True, verbose_name='created date'),
        ),
        migrations.AlterField(
            model_name='apply',
            name='email',
            field=models.CharField(blank=True, max_length=200, null=True, verbose_name='email'),
        ),
        migrations.AlterField(
            model_name='apply',
            name='project',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='ovp_projects.Project', verbose_name='project'),
        ),
        migrations.AlterField(
            model_name='apply',
            name='status',
            field=models.CharField(max_length=30, verbose_name='status'),
        ),
        migrations.AlterField(
            model_name='apply',
            name='user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='user'),
        ),
        migrations.AlterField(
            model_name='jobdate',
            name='job',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='dates', to='ovp_projects.Job', verbose_name='job'),
        ),
        migrations.AlterField(
            model_name='project',
            name='address',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='ovp_core.GoogleAddress', verbose_name='address'),
        ),
        migrations.AlterField(
            model_name='project',
            name='applied_count',
            field=models.IntegerField(default=0, verbose_name='Applied count'),
        ),
        migrations.AlterField(
            model_name='project',
            name='causes',
            field=models.ManyToManyField(to='ovp_core.Cause', verbose_name='causes'),
        ),
        migrations.AlterField(
            model_name='project',
            name='created_date',
            field=models.DateTimeField(auto_now_add=True, verbose_name='Created date'),
        ),
        migrations.AlterField(
            model_name='project',
            name='image',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='ovp_uploads.UploadedImage', verbose_name='image'),
        ),
        migrations.AlterField(
            model_name='project',
            name='modified_date',
            field=models.DateTimeField(auto_now=True, verbose_name='Modified date'),
        ),
        migrations.AlterField(
            model_name='project',
            name='organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='ovp_organizations.Organization', verbose_name='organization'),
        ),
        migrations.AlterField(
            model_name='project',
            name='owner',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='image'),
        ),
        migrations.AlterField(
            model_name='project',
            name='skills',
            field=models.ManyToManyField(to='ovp_core.Skill', verbose_name='skills'),
        ),
    ]
